/*******************************************************************************

  Defines Format Interface Utils for DIMIN Viewer n5 (DIV5)

  Programmer: Dima V. Fedorov Levit <dimin@dimin.net> <http://www.dimin.net/>

  History:
    06/23/2004 23:37 - First Creation

*******************************************************************************/

#ifndef DIV_FORMAT_UTILS_H
#define DIV_FORMAT_UTILS_H

#include "DIVFormatDefs.h"


TDivFormatHandle initTDivFormatHandle();
TDivImageInfo    initTDivImageInfo();

BYTE trimToByte(int val);

//------------------------------------------------------------------------------------------------
// SAFE CALLBACKS CALLS
//------------------------------------------------------------------------------------------------
void  showProgress ( TDivFormatHandle *fmtHndl, long done, long total, char *descr );
int   testAbort    ( TDivFormatHandle *fmtHndl );
void  showError    ( TDivFormatHandle *fmtHndl, int val, char *descr );
void* dimMalloc    ( TDivFormatHandle *fmtHndl, DIM_ULONG size );
void* dimFree      ( TDivFormatHandle *fmtHndl, void *p );

// the stream is specified by TDimFormatHandle
DIM_ULONG dimRead  ( TDivFormatHandle *fmtHndl, void *buffer, DIM_ULONG size, DIM_ULONG count );
DIM_ULONG dimWrite ( TDivFormatHandle *fmtHndl, void *buffer, DIM_ULONG size, DIM_ULONG count );
DIM_INT   dimFlush ( TDivFormatHandle *fmtHndl );
DIM_INT   dimSeek  ( TDivFormatHandle *fmtHndl, DIM_LONG offset, DIM_INT origin );
DIM_ULONG dimSize  ( TDivFormatHandle *fmtHndl );
DIM_LONG  dimTell  ( TDivFormatHandle *fmtHndl );
DIM_INT   dimEof   ( TDivFormatHandle *fmtHndl );
DIM_INT   dimClose ( TDivFormatHandle *fmtHndl );


//------------------------------------------------------------------------------------------------
// swap
//------------------------------------------------------------------------------------------------

void dimSwapShort(DIM_UINT16* wp);
void dimSwapLong(DIM_UINT32* lp);
void dimSwapArrayOfShort(DIM_UINT16* wp, register DIM_ULONG n);
void dimSwapArrayOfLong(register DIM_UINT32* lp, register DIM_ULONG n);
void dimSwapDouble(double *dp);
void dimSwapArrayOfDouble(double* dp, register DIM_ULONG n);
void dimSwapData(int type, long size, void* data);

//------------------------------------------------------------------------------
// PALETTE
//------------------------------------------------------------------------------

typedef DIM_UINT32 TDimRGBA;
typedef struct TDimLUT
{
  DIM_UINT count;
  TDimRGBA rgba[256]; 
} TDimLUT;

inline int dimR( TDimRGBA rgb )   
{ return (int)((rgb >> 16) & 0xff); }

inline int dimG( TDimRGBA rgb )   
{ return (int)((rgb >> 8) & 0xff); }

inline int dimB( TDimRGBA rgb )   
{ return (int)(rgb & 0xff); }

inline int dimA( TDimRGBA rgb )   
{ return (int)((rgb >> 24) & 0xff); }

inline TDimRGBA dimRGB( int r, int g, int b )
{ return (0xff << 24) | ((r & 0xff) << 16) | ((g & 0xff) << 8) | (b & 0xff); }

inline TDimRGBA dimRGBA( int r, int g, int b, int a )
{ return ((a & 0xff) << 24) | ((r & 0xff) << 16) | ((g & 0xff) << 8) | (b & 0xff); }

inline int dimGray( int r, int g, int b )
{ return (r*11+g*16+b*5)/32; }

inline int dimGray( TDimRGBA rgb )
{ return dimGray( dimR(rgb), dimG(rgb), dimB(rgb) ); }

HPALETTE createGreyPalette ( );
HPALETTE createHPalette ( int num_colors, RGBQUAD *bmiColors );


//------------------------------------------------------------------------------------------------
// image
//------------------------------------------------------------------------------------------------
DIM_UINT   getHBLineSizeInBytes( DIM_UINT w, DIM_UINT depth );
DIM_UCHAR *getHBScanLine( DIM_UINT y, DIM_UCHAR *p, DIM_UINT w, DIM_UINT depth );

// depth is in bits per pixel, return a pointer to image data area
int dimCreateHBitmap( TDivFormatHandle *fmtHndl, DIM_UINT w, DIM_UINT h, DIM_UINT depth, int num_colors, RGBQUAD *lut, DIM_UCHAR **p );


//------------------------------------------------------------------------------------------------
// metadata
//------------------------------------------------------------------------------------------------

void clearMetaTag( TDivTagItem *tagItem );
void clearMetaTag( TDivTagList *tagList, int pos );
void clearMetaTag( TDivTagList *tagList, int group, int tag );

void clearMetaTag( TDivFormatHandle *fmtHndl, TDivTagItem *tagItem );
void clearMetaTags( TDivFormatHandle *fmtHndl, TDivTagList *tagList );

void clearMetaTagsFreeTags( TDivFormatHandle *fmtHndl, TDivTagList *tagList );
bool isTagPresent(TDivTagList *tagList, int group, int tag);
int  tagPos(TDivTagList *tagList, int group, int tag);
int  addMetaTag( TDivFormatHandle *fmtHndl, TDivTagList *tagList, TDivTagItem item );

#endif //DIV_FORMAT_UTILS_H
