/*******************************************************************************

  FFMPEG (DLL) for DIMIN Viewer n5 (DIV5)

  Programmer: Dima V. Fedorov Levit <dimin@dimin.net> <http://www.dimin.net/>

  History:
    2008-04-13 10:58:00 - First Creation

*******************************************************************************/

#include <windows.h>

#include "div5_ffmpeg.h"
#include "src/dim_ffmpeg_format.h"

TDIVPluginHeader PluginHeader = {
  sizeof( TDIVPluginHeader ),
  5, // plugin interface version - MUST BE 5
  DIV_PLUGIN_TYPE_FORMAT,
  4, // plugin version
  1, // one format, this is basically not used for this type
  "DIMIN DIV5 FFMPEG movie Formats"
};

//------------------------------------------------------------------------------
// About procedure
//------------------------------------------------------------------------------
void __stdcall aboutPlugin( TDivFormatHandle *fmtHndl, HWND hwndParent ) 
{
  MessageBox(GetFocus(), "DIMIN Movie formats (FFMPEG)", "About Plug-in", MB_OK | MB_ICONEXCLAMATION);
}

//------------------------------------------------------------------------------
// Format definition
//------------------------------------------------------------------------------

#define DIM_FFMPEG_NUM_FORMATS 13

TDivFormatItem dimFFMpegItems[DIM_FFMPEG_NUM_FORMATS] = {
  {
    "MPEG",      // short name, no spaces
      "MPEG I", // Long format name
      "mpg|mpeg|m1v",        // pipe "|" separated supported extension list
      1, //canRead;      // 0 - NO, 1 - YES
      1, //canWrite;     // 0 - NO, 1 - YES
      0, //canReadMeta;  // 0 - NO, 1 - YES
      0, //canWriteMeta; // 0 - NO, 1 - YES
      1, //canWriteMultiPage;   // 0 - NO, 1 - YES
      //TDivFormatConstrains constrains ( w, h, pages, minsampl, maxsampl, minbitsampl, maxbitsampl, noLut )
    { 0, 0, 1, 3, 3, 8, 8, 1 } 
  },
  {
    "MPEG2",            // short name, no spaces
      "MPEG 2", // Long format name
      "mpg|mpeg|m2v|m2t",        // pipe "|" separated supported extension list
      1, //canRead;      // 0 - NO, 1 - YES
      1, //canWrite;     // 0 - NO, 1 - YES
      0, //canReadMeta;  // 0 - NO, 1 - YES
      0, //canWriteMeta; // 0 - NO, 1 - YES
      1, //canWriteMultiPage;   // 0 - NO, 1 - YES
      //TDivFormatConstrains constrains ( w, h, pages, minsampl, maxsampl, minbitsampl, maxbitsampl, noLut )
    { 0, 0, 1, 3, 3, 8, 8, 1 } 
  },
  {
    "MPEG4",            // short name, no spaces
      "MPEG 4", // Long format name
      "m4v",        // pipe "|" separated supported extension list
      1, //canRead;      // 0 - NO, 1 - YES
      1, //canWrite;     // 0 - NO, 1 - YES
      0, //canReadMeta;  // 0 - NO, 1 - YES
      0, //canWriteMeta; // 0 - NO, 1 - YES
      1, //canWriteMultiPage;   // 0 - NO, 1 - YES
      //TDivFormatConstrains constrains ( w, h, pages, minsampl, maxsampl, minbitsampl, maxbitsampl, noLut )
    { 0, 0, 1, 3, 3, 8, 8, 1 } 
  },
  {
    "MJPEG",            // short name, no spaces
      "Motion JPEG", // Long format name
      "mjpeg|mjp|mjpg",        // pipe "|" separated supported extension list
      1, //canRead;      // 0 - NO, 1 - YES
      1, //canWrite;     // 0 - NO, 1 - YES
      0, //canReadMeta;  // 0 - NO, 1 - YES
      0, //canWriteMeta; // 0 - NO, 1 - YES
      1, //canWriteMultiPage;   // 0 - NO, 1 - YES
      //TDivFormatConstrains constrains ( w, h, pages, minsampl, maxsampl, minbitsampl, maxbitsampl, noLut )
    { 0, 0, 1, 3, 3, 8, 8, 1 } 
  },
  {
    "MJPEG2000",            // short name, no spaces
      "Motion JPEG-2000", // Long format name
      "mj2|mjp2",        // pipe "|" separated supported extension list
      1, //canRead;      // 0 - NO, 1 - YES
      0, //canWrite;     // 0 - NO, 1 - YES
      0, //canReadMeta;  // 0 - NO, 1 - YES
      0, //canWriteMeta; // 0 - NO, 1 - YES
      0, //canWriteMultiPage;   // 0 - NO, 1 - YES
      //TDivFormatConstrains constrains ( w, h, pages, minsampl, maxsampl, minbitsampl, maxbitsampl, noLut )
    { 0, 0, 1, 3, 3, 8, 8, 1 } 
  },
  {
    "AVI",            // short name, no spaces
      "Microsoft Windows AVI", // Long format name
      "avi",        // pipe "|" separated supported extension list
      1, //canRead;      // 0 - NO, 1 - YES
      1, //canWrite;     // 0 - NO, 1 - YES
      0, //canReadMeta;  // 0 - NO, 1 - YES
      0, //canWriteMeta; // 0 - NO, 1 - YES
      1, //canWriteMultiPage;   // 0 - NO, 1 - YES
      //TDivFormatConstrains constrains ( w, h, pages, minsampl, maxsampl, minbitsampl, maxbitsampl, noLut )
    { 0, 0, 1, 3, 3, 8, 8, 1 } 
  },
  {
    "WMV",            // short name, no spaces
      "Microsoft Windows Media Video", // Long format name
      "wmv|asf",        // pipe "|" separated supported extension list
      1, //canRead;      // 0 - NO, 1 - YES
      1, //canWrite;     // 0 - NO, 1 - YES
      0, //canReadMeta;  // 0 - NO, 1 - YES
      0, //canWriteMeta; // 0 - NO, 1 - YES
      1, //canWriteMultiPage;   // 0 - NO, 1 - YES
      //TDivFormatConstrains constrains ( w, h, pages, minsampl, maxsampl, minbitsampl, maxbitsampl, noLut )
    { 0, 0, 1, 3, 3, 8, 8, 1 } 
  },
  {
    "QuickTime",            // short name, no spaces
      "Apple QuickTime", // Long format name
      "mov",        // pipe "|" separated supported extension list
      1, //canRead;      // 0 - NO, 1 - YES
      1, //canWrite;     // 0 - NO, 1 - YES
      0, //canReadMeta;  // 0 - NO, 1 - YES
      0, //canWriteMeta; // 0 - NO, 1 - YES
      1, //canWriteMultiPage;   // 0 - NO, 1 - YES
      //TDivFormatConstrains constrains ( w, h, pages, minsampl, maxsampl, minbitsampl, maxbitsampl, noLut )
    { 0, 0, 1, 3, 3, 8, 8, 1 } 
  },
  {
    "Flash",            // short name, no spaces
      "Adobe Flash", // Long format name
      "swf",        // pipe "|" separated supported extension list
      0, //canRead;      // 0 - NO, 1 - YES
      1, //canWrite;     // 0 - NO, 1 - YES
      0, //canReadMeta;  // 0 - NO, 1 - YES
      0, //canWriteMeta; // 0 - NO, 1 - YES
      1, //canWriteMultiPage;   // 0 - NO, 1 - YES
      //TDivFormatConstrains constrains ( w, h, pages, minsampl, maxsampl, minbitsampl, maxbitsampl, noLut )
    { 0, 0, 1, 3, 3, 8, 8, 1 } 
  },
  {
    "OGG",            // short name, no spaces
      "OGG Theora Video Format", // Long format name
      "ogg|ogv",        // pipe "|" separated supported extension list
      1, //canRead;      // 0 - NO, 1 - YES
      0, //canWrite;     // 0 - NO, 1 - YES
      0, //canReadMeta;  // 0 - NO, 1 - YES
      0, //canWriteMeta; // 0 - NO, 1 - YES
      0, //canWriteMultiPage;   // 0 - NO, 1 - YES
      //TDivFormatConstrains constrains ( w, h, pages, minsampl, maxsampl, minbitsampl, maxbitsampl, noLut )
    { 0, 0, 1, 3, 3, 8, 8, 1 } 
  },
  {
    "Matroska",            // short name, no spaces
      "Matroska Multimedia Container", // Long format name
      "mkv",        // pipe "|" separated supported extension list
      1, //canRead;      // 0 - NO, 1 - YES
      1, //canWrite;     // 0 - NO, 1 - YES
      0, //canReadMeta;  // 0 - NO, 1 - YES
      0, //canWriteMeta; // 0 - NO, 1 - YES
      1, //canWriteMultiPage;   // 0 - NO, 1 - YES
      //TDivFormatConstrains constrains ( w, h, pages, minsampl, maxsampl, minbitsampl, maxbitsampl, noLut )
    { 0, 0, 1, 3, 3, 8, 8, 1 } 
  },
  {
    "DV",            // short name, no spaces
      "Digital Video", // Long format name
      "dv",        // pipe "|" separated supported extension list
      1, //canRead;      // 0 - NO, 1 - YES
      0, //canWrite;     // 0 - NO, 1 - YES
      0, //canReadMeta;  // 0 - NO, 1 - YES
      0, //canWriteMeta; // 0 - NO, 1 - YES
      0, //canWriteMultiPage;   // 0 - NO, 1 - YES
      //TDivFormatConstrains constrains ( w, h, pages, minsampl, maxsampl, minbitsampl, maxbitsampl, noLut )
    { 0, 0, 1, 3, 3, 8, 8, 1 } 
  },
  {
    "FLV",            // short name, no spaces
      "Adobe Flash Video", // Long format name
      "flv",        // pipe "|" separated supported extension list
      0, //canRead;      // 0 - NO, 1 - YES
      1, //canWrite;     // 0 - NO, 1 - YES
      0, //canReadMeta;  // 0 - NO, 1 - YES
      0, //canWriteMeta; // 0 - NO, 1 - YES
      1, //canWriteMultiPage;   // 0 - NO, 1 - YES
      //TDivFormatConstrains constrains ( w, h, pages, minsampl, maxsampl, minbitsampl, maxbitsampl, noLut )
    { 0, 0, 1, 1, 3, 1, 8, 0 } 
  }                
};

TDivFormatHeader formatHeader = {
  sizeof(TDivFormatHeader),      // must be == sizeof(TDivFormatHeader)
  "1.0.1",                  // plugin version, e.g. "1.0.0"
  "FFMPEG Codec",       // plugin name
  "Movie formats codec for div5",    // plugin description
  
  20, //8, // 0 or more, specify number of bytes needed to identify the file // 8 - all, 56 - BioRad
  {1, DIM_FFMPEG_NUM_FORMATS, dimFFMpegItems}, //TDivFormatList              supportedFormats;
  
  dimFFMpegValidateFormatProc, //TDivValidateFormatProc      validateFormatProc; // return TRUE if can handle file
  // begin
  dimFFMpegAquireFormatProc, //TDivAquireFormatProc        aquireFormatProc;
  // end
  dimFFMpegReleaseFormatProc, //TDivReleaseFormatProc       releaseFormatProc;

  // params
  aboutPlugin, //TDivAboutFormatProc         aboutFormatProc;
  NULL, //TDivConfigFormatProc        configFormatProc;

  // image begin
  dimFFMpegOpenImageProc, //TDivOpenImageProc           openImageProc; 
  dimFFMpegCloseImageProc, //TDivCloseImageProc          closeImageProc; 

  // info
  dimFFMpegGetNumPagesProc, //TDivGetNumPagesProc         getNumPagesProc;
  dimFFMpegGetImageInfoProc, //TDivGetImageInfoProc        getImageInfoProc;

  // read/write
  dimFFMpegReadImageProc, //TDivReadImageProc           readImageProc; 
  dimFFMpegWriteImageProc, //TDivWriteImageProc          writeImageProc;
  NULL, //TDivReadImageTileProc       readImageTileProc;
  NULL, //TDivWriteImageTileProc      writeImageTileProc;
  NULL, //TDivReadImageLineProc       readImageLineProc;
  NULL, //TDivWriteImageLineProc      writeImageLineProc;
  NULL, //TDivReadImageThumbProc      readImageThumbProc;
  NULL, //TDivWriteImageThumbProc     writeImageThumbProc;
  NULL, //TDivReadImagePreviewProc    readImagePreviewProc;
  
  // meta data
  NULL, //TDivReadMetaDataProc        readMetaDataProc;
  NULL, //TDivAddMetaDataProc         addMetaDataProc;
  dimFFMpegReadMetaDataAsTextProc, //TDivReadMetaDataAsTextProc  readMetaDataAsTextProc;

  dimFFMpegConfigInitProc, //TDivConfigFormatProc
  dimFFMpegConfigQuitProc, //TDivConfigFormatProc

  // xtd
  NULL, // xtdProc

  NULL, // p1
  NULL, // p2
  NULL, // p3

};

//---------------------------------------------------------------------------

BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
  switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
    }
    return TRUE;
}


TDIVPluginHeader* __stdcall divGetPluginHeader()
{
  return &PluginHeader;
}

TDivFormatHeader* __stdcall divGetFormatHeader()
{
   return &formatHeader;
}



//---------------------------------------------------------------------------
