/*******************************************************************************
  
  Map for tag/value pairs
    
  Author: Dima Fedorov Levit <dimin@dimin.net> <http://www.dimin.net/>

  History:
    2008-03-18 17:13 - First creation
      
  ver: 1
        
*******************************************************************************/

#include "tag_map.h"
#include "xstring.h"

#include <string>
#include <sstream>
#include <map>

//******************************************************************************

void DTagMap::append_tag( const std::string &key, const std::string &value ) {
  this->insert( make_pair( key, value ) );
}

void DTagMap::append_tag( const std::string &key, const int &value ) {
  xstring val;
  val.sprintf("%d", value);
  this->insert( make_pair( key, val ) );
}

void DTagMap::append_tag( const std::string &key, const double &value ) {
  xstring val;
  val.sprintf("%f", value);
  this->insert( make_pair( key, val ) );
}

//******************************************************************************

std::string DTagMap::get_value( const std::string &key, const std::string &def ) const {
  std::string v = def;
  std::map<std::string, std::string>::const_iterator it = this->find( key ); 
  if (it != this->end() )
    v = (*it).second;
  return v;
}

int DTagMap::get_value_int( const std::string &key, const int &def ) const {
  xstring str = this->get_value( key, "" );
  return str.toInt( def );
}

double DTagMap::get_value_double( const std::string &key, const double &def ) const {
  xstring str = this->get_value( key, "" );
  return str.toDouble( def );
}



