/*****************************************************************************
 Extended String Class

 IMPLEMENTATION

 Author: Dima V. Fedorov <mailto:dima@dimin.net> <http://www.dimin.net/>

 History:
   12/05/2005 23:38 - First creation

 Ver : 5
*****************************************************************************/

#include <cstdarg>
#include <cstdio>

#include <sstream>
#include <algorithm>

#include "xstring.h"

const int MAX_STR_SIZE = 1024;

xstring &xstring::sprintf(const char *fmt, ...) {
  va_list ap;
  va_start(ap, fmt);
  sprintf( fmt, ap );
  va_end(ap);
  return *this;
}

xstring &xstring::saddf(const char *fmt, ...) {
  xstring result;
  va_list ap;
  va_start(ap, fmt);
  result.sprintf( fmt, ap );
  va_end(ap);
  *this += result;
  return ( *this );
}

xstring xstring::xprintf(const char *fmt, ...) {
  xstring result;
  va_list ap;
  va_start(ap, fmt);
  result.sprintf( fmt, ap );
  va_end(ap);
  return result;
}

xstring &xstring::sprintf( const char *format, va_list ap ) {
  xstring result;
  char cbuf[MAX_STR_SIZE];

#ifdef HAVE_SECURE_C  
  vsprintf_s((char *) cbuf, MAX_STR_SIZE, format, ap);
#else
  vsprintf((char *) cbuf, format, ap);
#endif
  result += cbuf;

  *this = result;
  return *this;
}

//******************************************************************************

xstring &xstring::insertAfterLast(const char *p, const xstring &s) {
  unsigned int sp = (unsigned int) this->rfind(p);
  if (sp != std::string::npos)
    this->insert(sp, s);
  return *this;
}

//******************************************************************************

xstring &xstring::removeSpacesLeft() {
  int p = this->find_first_not_of(" ");
  if (p != std::string::npos) {
    *this = this->substr( p, std::string::npos );
  }
  return *this;
}

xstring &xstring::removeSpacesRight() {
  int p = this->find_last_not_of(" ");
  if (p != std::string::npos) {
    this->resize(p+1, ' ');
  }
  return *this;
}

xstring &xstring::removeSpacesBoth() {
  removeSpacesLeft();
  removeSpacesRight();
  return *this;
}

//******************************************************************************

template<typename RT, typename T, typename Trait, typename Alloc>
RT ss_atoi( const std::basic_string<T, Trait, Alloc>& the_string ) {
  std::basic_istringstream< T, Trait, Alloc> temp_ss(the_string);
  RT num;
  temp_ss >> num;
  return num;
}

int xstring::toInt( int def ) {
  int v = def;
  if (this->size()>0) v = ss_atoi<int>(*this);
  return v;
}

double xstring::toDouble( double def ) {
  double v = def;
  if (this->size()>0) v = ss_atoi<double>(*this);
  return v;
}

//******************************************************************************

bool xstring::operator==(const xstring &other) const {
  return ( size() == other.size()) && (memcmp( this->c_str(), other.c_str(), size() )==0);
}

bool xstring::operator==(const char *s) const {
  xstring other = s;
  return ( size() == other.size()) && (memcmp( this->c_str(), other.c_str(), size() )==0);
}

int xstring::compare(const xstring &s) const {
  return strncmp( this->c_str(), s.c_str(), std::min(this->size(), s.size()) );
}

//******************************************************************************
std::vector<xstring> xstring::split( const xstring &s ) const {

  std::vector<xstring> list;
  xstring part;

  std::string::size_type start = 0;
  std::string::size_type end;

  while (1) {
    end = this->find ( s, start );
    if (end == std::string::npos) {
      part = this->substr( start );
      list.push_back( part );
      break;
    } else {
      part = this->substr( start, end-start );
      start = end + s.size();
      list.push_back( part );
    }
  }
 
  return list;
}
