/*******************************************************************************

  Memory IO
 
  Programmer: Dima Fedorov Levit <dimin@dimin.net> <http://www.dimin.net/>
  
  Based on memio used by GDAL and libtiff by:
      Mike Johnson - Banctec AB
      Frank Warmerdam, warmerdam@pobox.com

  History:
    08/05/2004 13:57 - First creation
      
  ver: 1
        
*******************************************************************************/

#ifndef DIM_MEMIO_H
#define DIM_MEMIO_H

#if defined(__cplusplus)
extern "C" {
#endif

#ifndef ulong
#define ulong unsigned long
#endif

#ifndef uchar
#define uchar unsigned char
#endif

typedef struct TDimMemIO {
    unsigned char *data;         // allocated data buffer
    unsigned long size;          // "file" size in bytes
    unsigned long data_buf_size; // in bytes - may be larger than used
    unsigned long offset;        // current file offset from start of file
    int           own_buffer;    // non-zero if 'data' is owned by MemIOBuf
} TDimMemIO;


void  dimMemIO_Init( TDimMemIO *, int size, unsigned char *data );
void  dimMemIO_Destroy( TDimMemIO * );

ulong dimMemIO_Read  ( void *buffer, ulong size, ulong count, void *stream );
ulong dimMemIO_Write ( void *buffer, ulong size, ulong count, void *stream );
int   dimMemIO_Seek  ( void *stream, long offset, int origin );
int   dimMemIO_Flush ( void *stream );
ulong dimMemIO_Size  ( void *stream );
long  dimMemIO_Tell  ( void *stream );
int   dimMemIO_Eof   ( void *stream );
int   dimMemIO_Close ( void *stream );


#if defined(__cplusplus)
}
#endif

#endif // DIM_MEMIO_H
