/*******************************************************************************

  Manager for Image Formats with MetaData parsing

  Uses DimFiSDK version: 1
  
  Programmer: Dima Fedorov Levit <dimin@dimin.net> <http://www.dimin.net/>

  Notes:
    Session: during session any session wide operation will be performed
    with current session!!!
    If you want to start simultaneous sessions then create new manager
    using = operator. It will copy all necessary initialization data
    to the new manager.

    Non-session wide operation might be performed simultaneously with 
    an open session.

  History:
    03/23/2004 18:03 - First creation
    01/25/2007 21:00 - added QImaging TIFF
      
  ver: 2
        
*******************************************************************************/

#ifndef META_FORMAT_MANAGER_H
#define META_FORMAT_MANAGER_H

#include "dim_format_manager.h"

#include <vector>
#include <map>
#include "tag_map.h"

class TMetaFormatManager : public TDimFormatManager
{
public:
  TMetaFormatManager();
  ~TMetaFormatManager();
  int  sessionStartRead (const char *fileName);
  int  sessionReadImage ( TDimImageBitmap *bmp, DIM_UINT page );

  void sessionParseMetaData ( DIM_UINT page );
  TDimImageBitmap *sessionImage();
  void sessionEnd();

  // META
  double       getPixelSizeX();
  double       getPixelSizeY();
  double       getPixelSizeZ();
  double       getPixelSizeT();
  const char*  getImagingTime();

  inline const std::string                        &get_text_metadata() const { return meta_data_text; }  
  inline const std::map<std::string, std::string> &get_metadata() const      { return metadata; }
  inline const std::vector< int >                 &get_display_lut() const   { return display_lut; }
  inline const std::vector< std::string >         &get_channel_names() const { return channel_names; }

  inline std::string get_metadata_tag( const std::string &key, const std::string &def ) const { metadata.get_value( key, def ); }
  inline int         get_metadata_tag_int( const std::string &key, const int &def ) const { metadata.get_value_int( key, def ); }
  inline double      get_metadata_tag_double( const std::string &key, const double &def ) const { metadata.get_value_double( key, def ); }

private:
  int          got_meta_for_session;
  TDimTagList *tagList;
  std::string  meta_data_text;

  // key-value pairs for metadata
  DTagMap metadata;
  inline void appendMetadata( const std::string &key, const std::string &value ) { metadata.append_tag( key, value ); }
  inline void appendMetadata( const std::string &key, const int &value ) { metadata.append_tag( key, value ); }
  inline void appendMetadata( const std::string &key, const double &value ) { metadata.append_tag( key, value ); }

  // META
  double pixel_size[4]; //XYZ in microns, T in seconds
  char   imaging_time[20]; // "YYYY-MM-DD HH:MM:SS" ANSI date and 24h time
  std::vector< std::string > channel_names;
  std::vector< int > display_lut;

  void parse_fluoview_285 (char *buf);
  void parse_fluoview_270 (char *buf);
  void parse_biorad_01 (char *buf);
  void parse_biorad_20 (char *buf);
  void parse_stk_33629 (char *buf);
  
  void parse_nanoscope();
  void parse_biorad();
  void parse_stk();
  void parse_fluoview();
  void parse_psia();
  int  parse_qimagingtiff();
  void parse_gentiff();
  void parse_ibw();
  void parse_lsm();

  void parse_meta_data();

};

#endif // META_FORMAT_MANAGER_H

