/*****************************************************************************
  Carl Zeiss LSM definitions 
  Copyright (c) 2006 by Dmitry V. Fedorov <www.dimin.net> <dima@dimin.net>

  IMPLEMENTATION
  
  Programmer: Dima V. Fedorov <mailto:dima@dimin.net> <http://www.dimin.net/>
    
  History:
    03/29/2004 22:23 - First creation

  Ver : 1
*****************************************************************************/

#ifndef DIM_CZ_LSM_FORMAT_H
#define DIM_CZ_LSM_FORMAT_H

#include <dim_img_format_interface.h>

//----------------------------------------------------------------------------
// Internal LSM Structures
//----------------------------------------------------------------------------

#define UINT32 DIM_UINT32
#define SINT32 DIM_INT32
#define FLOAT64 DIM_FLOAT64


#pragma pack(push, 1)
typedef struct CZ_LSMINFO {
  UINT32   u32MagicNumber;
  SINT32   s32StructureSize;
  SINT32   s32DimensionX;              
  SINT32   s32DimensionY;
  SINT32   s32DimensionZ;
  SINT32   s32DimensionChannels;
  SINT32   s32DimensionTime;
  SINT32   s32DataType;                
  SINT32   s32ThumbnailX;               
  SINT32   s32ThumbnailY;
  FLOAT64  f64VoxelSizeX;               
  FLOAT64  f64VoxelSizeY;               
  FLOAT64  f64VoxelSizeZ;              
  UINT32   u32ScanType;
  UINT32   u32DataType;
  UINT32   u32OffsetVectorOverlay;
  UINT32   u32OffsetInputLut;
  UINT32   u32OffsetOutputLut;
  UINT32   u32OffsetChannelColors;
  FLOAT64  f64TimeIntervall;
  UINT32   u32OffsetChannelDataTypes;
  UINT32   u32OffsetScanInformation;
  UINT32   u32OffsetKsData;
  UINT32   u32OffsetTimeStamps; 
  UINT32   u32OffsetEventList;
  UINT32   u32OffsetRoi;
  UINT32   u32OffsetBleachRoi;
  UINT32   u32OffsetNextRecording;
  UINT32   u32Reserved [ 90 ];
} CZ_LSMINFO;
#pragma pack(pop)

#pragma pack(push, 1)
typedef struct CZ_ChannelColors {
  SINT32 s32BlockSize;    // Size of the structure in bytes including the name strings and colors.
  SINT32 s32NumberColors; // Number of colors in the color array; should be the same as the number of channels.
  SINT32 s32NumberNames;  // Number of character strings for the channel names; should be the same as the number of channels.
  SINT32 s32ColorsOffset; // Offset relative ti the start of the structure to the UINT32 array of channel colors. 
                          // Each array entry contains a color with intensity values in the range 0..255 for the three color components
  SINT32 s32NamesOffset;  // Offset relative ti the start of the structure to the list of channel names. The list of channel names is a series of \0-terminated ANSI character strings.
  SINT32 s32Mono;         // If unequal zero the Mono button in the LSM-imagefenster  window was peressed
} CZ_ChannelColors;
#pragma pack(pop)

//----------------------------------------------------------------------------
// TIFF Codec-wise Fluoview Structure 
//----------------------------------------------------------------------------
typedef struct TDimLsmInfo
{
  int ch;        // number of channels in each image of the sequence
  int z_slices;  // number of Z slices for each time instance
  int t_frames;  // number of time frames in the sequence
  int pages;     // the value of consequtive images generated by driver
  double res[4]; // pixel resolution for XYZT
  CZ_LSMINFO lsm_info; // actual header structure retreived form the image
  CZ_ChannelColors lsm_colors;

  //internal
  int pages_tiff;
} TDimLsmInfo;


#undef UINT32
#undef SINT32
#undef FLOAT64

#endif // DIM_CZ_LSM_FORMAT_H

