/*****************************************************************************
  TIFF support 
  Copyright (c) 2004 by Dmitry V. Fedorov <www.dimin.net> <dima@dimin.net>

  IMPLEMENTATION
  
  Programmer: Dima V. Fedorov <mailto:dima@dimin.net> <http://www.dimin.net/>

  Notes:
    Metadata can be red using readMetaData but can not br written to the file
    using addMetaData, it must be supplied with the formatHandler within
    writeImage function.

  History:
    03/29/2004 22:23 - First creation
        
  Ver : 1
*****************************************************************************/

#ifndef DIM_TIFF_FORMAT_H
#define DIM_TIFF_FORMAT_H

#include <dim_img_format_interface.h>
#include <dim_img_format_utils.h>

#include "dim_tiny_tiff.h"
#include "dim_psia_format.h"
#include "dim_stk_format.h"
#include "dim_fluoview_format.h"
#include "dim_cz_lsm_format.h"

#ifndef XMD_H
#define XMD_H // Shut JPEGlib up.
#endif  
  
#include <tiffio.h>
#include <tif_dir.h>

typedef enum {
  tstGeneric  = 0,
  tstStk      = 1,
  tstPsia     = 2,
  tstFluoview = 3,
  tstCzLsm    = 4
} DIM_TiffSubType;

typedef struct TDimTiffParams
{
  TIFF  *dimTiff;
  DIM_TiffSubType subType;
  TDimImageInfo info;
  TDimTiffIFDs ifds;
  TDimStkInfo stkInfo;
  psiaInfoHeader psiaInfo;
  TDimFluoviewInfo fluoviewInfo;
  TDimLsmInfo lsmInfo;
} TDimTiffParams;


// DLL EXPORT FUNCTION
extern "C" {
TDimFormatHeader* dimTiffGetFormatHeader(void);
}

// INTERNAL FUNCTIONS
DIM_INT dimTiffValidateFormatProc (DIM_MAGIC_STREAM *magic, DIM_UINT length);

TDimFormatHandle dimTiffAquireFormatProc( void );
void dimTiffReleaseFormatProc (TDimFormatHandle *fmtHndl);

DIM_UINT dimTiffOpenImageProc  (TDimFormatHandle *fmtHndl, DIM_ImageIOModes io_mode);
DIM_UINT dimTiffFOpenImageProc (TDimFormatHandle *fmtHndl, char* fileName, DIM_ImageIOModes io_mode);
DIM_UINT dimTiffIOpenImageProc (TDimFormatHandle *fmtHndl, char* fileName, 
                                         DIM_IMAGE_CLASS *image, DIM_ImageIOModes io_mode);

void dimTiffCloseImageProc     (TDimFormatHandle *fmtHndl);

DIM_UINT dimTiffGetNumPagesProc       ( TDimFormatHandle *fmtHndl );
TDimImageInfo dimTiffGetImageInfoProc ( TDimFormatHandle *fmtHndl, DIM_UINT page_num );

DIM_UINT dimTiffReadImageProc  (TDimFormatHandle *fmtHndl, DIM_UINT page);
DIM_UINT dimTiffWriteImageProc (TDimFormatHandle *fmtHndl);

DIM_UINT dimTiffReadImagePreviewProc (TDimFormatHandle *fmtHndl, DIM_UINT w, DIM_UINT h);

DIM_UINT dimTiffReadMetaDataProc (TDimFormatHandle *fmtHndl, DIM_UINT page, int group, int tag, int type);
char* dimTiffReadMetaDataAsTextProc ( TDimFormatHandle *fmtHndl );

#endif // DIM_TIFF_FORMAT_H
