/*******************************************************************************

  Implementation of the Image Class for Trolltech Qt
  
  Author: Dima Fedorov Levit <dimin@dimin.net> <http://www.dimin.net/>

  History:
    03/24/2006 12:45 - First creation
      
  ver: 1
        
*******************************************************************************/

#include "dim_image.h"
#include "dim_img_format_utils.h"

#ifdef DIM_USE_QT

#pragma message("TDimImage: Trolltech Qt support methods")

QImage  TDimImage::toQImage ( ) {
  int max_samples = bmp->i.samples; 
  if (max_samples > 3) max_samples = 3;
  DIM_UCHAR lut[3][65536];

  //------------------------------------------------------------------
  // if input image is 16 bpp/ch then normalize to 8 bpp/ch
  //------------------------------------------------------------------
  if (bmp->i.depth == 16) {
    DIM_UINT max_uint16 = (DIM_UINT16) -1;
    long hist[65536];
    DIM_UINT min_col = 0;
    DIM_UINT max_col = max_uint16;
    DIM_INT32 i;
    int range=0;
    DIM_UINT sample=0;

    for (sample=0; sample<max_samples; sample++) {
      DIM_UINT16 *p16 = (DIM_UINT16 *) bmp->bits[sample];
      getSampleHistogram((TDimImageBitmap *) &bmp, hist, sample);

      for (i=0; i<=max_uint16; i++) {
        if (hist[i] != 0) 
        { min_col = i; break; }
      }
  
      for (i=max_uint16; i>=0; i--) {
        if (hist[i] != 0) 
        { max_col = i; break; }
      }

      range = (max_col - min_col) / 256.0;
      if (range == 0) range = 256;

      for (i=0; i<=max_uint16; i++)  
        lut[sample][i] = iTrimUC ( (i - min_col) / range );
    }

  } // if 16 bpp then normalize

  //------------------------------------------------------------------
  // Create QImage and copy data
  //------------------------------------------------------------------
  QImage::Format imageFormat = QImage::Format_ARGB32_Premultiplied;
  QImage image(bmp->i.width, bmp->i.height, imageFormat);

  for (int y=0; y<bmp->i.height; ++y) {
    QRgb *dest = (QRgb *) image.scanLine(y);

    unsigned int src_line_offset = bmp->i.width * y;

    if (max_samples == 1) {
  
      if (bmp->i.depth == 8) {
        DIM_UCHAR *src1 = ((DIM_UCHAR *) bmp->bits[0]) + src_line_offset;

        if (bmp->i.lut.count <= 0) { // no LUT
          for (int x=0; x<bmp->i.width; ++x) 
            dest[x] = qRgb ( src1[x], src1[x], src1[x] );
        }
        else { // use LUT to create colors
          TDimRGBA *pal = (TDimRGBA *) bmp->i.lut.rgba;

          for (int x=0; x<bmp->i.width; ++x) 
            dest[x] = qRgb ( dimR(pal[src1[x]]), dimG(pal[src1[x]]), dimB(pal[src1[x]]) );
        }
      } // 8 bpp
    
      if (bmp->i.depth == 16) {
        DIM_UINT16 *src0 = ((DIM_UINT16 *) bmp->bits[0]) + src_line_offset;

        if (bmp->i.lut.count <= 0) { // no LUT
          for (int x=0; x<bmp->i.width; ++x) 
            dest[x] = qRgb ( lut[0][src0[x]], lut[0][src0[x]], lut[0][src0[x]] );
        }
        else { // use LUT to create colors
          TDimRGBA *pal = (TDimRGBA *) bmp->i.lut.rgba;

          for (int x=0; x<bmp->i.width; ++x) 
            dest[x] = qRgb ( dimR(pal[lut[0][src0[x]]]), dimG(pal[lut[0][src0[x]]]), dimB(pal[lut[0][src0[x]]]) );
        }
      } // 16 bpp      

    } // if (max_samples == 1) 


    if (max_samples == 2) {
  
      if (bmp->i.depth == 8) {
        DIM_UCHAR *src0 = ((DIM_UCHAR *) bmp->bits[0]) + src_line_offset;
        DIM_UCHAR *src1 = ((DIM_UCHAR *) bmp->bits[1]) + src_line_offset;

        for (int x=0; x<bmp->i.width; ++x) 
          dest[x] = qRgb ( src0[x], src1[x], 0 );
      } // 8 bpp
    
      if (bmp->i.depth == 16) {
        DIM_UINT16 *src0 = ((DIM_UINT16 *) bmp->bits[0]) + src_line_offset;
        DIM_UINT16 *src1 = ((DIM_UINT16 *) bmp->bits[1]) + src_line_offset;

        for (int x=0; x<bmp->i.width; ++x) 
          dest[x] = qRgb ( lut[0][src0[x]], lut[1][src1[x]], 0 );
      } // 16 bpp      

    } // if (max_samples == 2) 

    if (max_samples >= 3) {
      if (bmp->i.depth == 8) {
        DIM_UCHAR *src0 = ((DIM_UCHAR *) bmp->bits[0]) + src_line_offset;
        DIM_UCHAR *src1 = ((DIM_UCHAR *) bmp->bits[1]) + src_line_offset;
        DIM_UCHAR *src2 = ((DIM_UCHAR *) bmp->bits[2]) + src_line_offset;

        for (int x=0; x<bmp->i.width; ++x) 
          dest[x] = qRgb ( src0[x], src1[x], src2[x] );
      } // 8 bpp
    
      if (bmp->i.depth == 16) {
        DIM_UINT16 *src0 = ((DIM_UINT16 *) bmp->bits[0]) + src_line_offset;
        DIM_UINT16 *src1 = ((DIM_UINT16 *) bmp->bits[1]) + src_line_offset;
        DIM_UINT16 *src2 = ((DIM_UINT16 *) bmp->bits[2]) + src_line_offset;

        for (int x=0; x<bmp->i.width; ++x) 
          dest[x] = qRgb ( lut[0][src0[x]], lut[1][src1[x]], lut[2][src2[x]] );
      } // 16 bpp      

    } // if (max_samples == 3) 


  } // for y

  return image;
}

QPixmap TDimImage::toQPixmap ( )
{
  QImage image = this->toQImage();
  return QPixmap::fromImage( image );
}

void TDimImage::fromQImage( const QImage &qimg ) {

  if (qimg.depth() < 32) return;
  
  if ( qimg.width() != width() || qimg.height() != height() )
    this->alloc( qimg.width(), qimg.height(), 3, 8 );

  //------------------------------------------------------------------
  // Create QImage and copy data
  //------------------------------------------------------------------
  unsigned char *p0 = this->scanLine( 0, 0 );
  unsigned char *p1 = this->scanLine( 1, 0 );
  unsigned char *p2 = this->scanLine( 2, 0 );
  const unsigned char *pI = qimg.bits();  
  
  for (unsigned int y=0; y<height(); ++y) {
    for (unsigned int x=0; x<width(); ++x) {
      #ifndef Q_WS_WIN
      *p0 = pI[1];
      *p1 = pI[1];
      *p2 = pI[2];
      #else
      *p0 = pI[2];
      *p1 = pI[1];
      *p2 = pI[0];
      #endif
      pI+=4; ++p0; ++p1; ++p2;
    }
  } // for y
  
}


#endif //DIM_USE_QT























