/*****************************************************************************
FFMPEG support 
Copyright (c) 2008 by Dmitry V. Fedorov <www.dimin.net> <dima@dimin.net>

IMPLEMENTATION

Programmer: Dima V. Fedorov <mailto:dima@dimin.net> <http://www.dimin.net/>

History:
2008-02-01 14:45 - First creation

Ver : 1
*****************************************************************************/

#include <cstdio>
#include <cstdlib>
#include <cstring>

#include <string>
#include <set>
#include <list>
#include <utility>

#include <xstring.h>

#include "dim_ffmpeg_format.h"
#include "dim_ffmpeg_format_io.cpp"

// Disables Visual Studio 2005 warnings for deprecated code
#if ( defined(_MSC_VER) && (_MSC_VER >= 1400) )
#pragma warning(disable:4996)
#endif 


//****************************************************************************
// INTERNAL STRUCTURES
//****************************************************************************

bool ffmpegGetImageInfo( TDivFormatHandle *fmtHndl ) {

  if (fmtHndl == NULL) return FALSE;
  if (fmtHndl->internalParams == NULL) return FALSE;
  TDimFFMpegParams *par = (TDimFFMpegParams *) fmtHndl->internalParams;
  TDivImageInfo *info = &par->i;  
  *info = initTDivImageInfo();

  if (par->ff_in.numFrames() <= 0) return false;

  info->width        = par->ff_in.width();
  info->height       = par->ff_in.height();
  info->depth        = 8;
  info->samples      = par->ff_in.depth();
  info->number_pages = par->ff_in.numFrames();
  //info->imageMode    = DIM_RGB;

  const char *fmt_name = par->ff_in.formatName();
  const char *cdc_name = par->ff_in.codecName();  

  std::map<std::string, int>::const_iterator it = formats.find( fmt_name );
  if (it != formats.end()) {
    fmtHndl->subFormat = (*it).second;
  }

  return true;
}

//****************************************************************************
// FORMAT DEMANDED FUNTIONS
//****************************************************************************


//----------------------------------------------------------------------------
// PARAMETERS, INITS
//----------------------------------------------------------------------------

DIM_INT __stdcall dimFFMpegValidateFormatProc (void *magic, DIM_UINT32 length) {
  if (length < 16) return -1;
  unsigned char *mag_num = (unsigned char *) magic;

  //Hex: 00 00 01 Bx
  if ( (mag_num[0] == 0x00) && (mag_num[1] == 0x00) && (mag_num[2] == 0x01) && ( (mag_num[3]>>4) == 0xB) ) return 0; // mpeg 1,2,4

  if ( (mag_num[0] == 0x52) && (mag_num[1] == 0x49) && (mag_num[2] == 0x46) && (mag_num[3] == 0x46) &&
    (mag_num[8] == 0x41) && (mag_num[9] == 0x56) && (mag_num[10] == 0x49) ) return 5; // avi

  if ( (mag_num[4] == 0x6D) && (mag_num[5] == 0x6F) && (mag_num[6] == 0x6F) && (mag_num[7] == 0x76) ) return 7; // qt mov (moov)
  if ( (mag_num[4] == 0x66) && (mag_num[5] == 0x72) && (mag_num[6] == 0x65) && (mag_num[7] == 0x65) ) return 7; // qt mov (free)
  if ( (mag_num[4] == 0x6D) && (mag_num[5] == 0x64) && (mag_num[6] == 0x61) && (mag_num[7] == 0x74) ) return 7; // qt mov (mdat)
  if ( (mag_num[4] == 0x77) && (mag_num[5] == 0x69) && (mag_num[6] == 0x64) && (mag_num[7] == 0x65) ) return 7; // qt mov (wide)
  if ( (mag_num[4] == 0x70) && (mag_num[5] == 0x6E) && (mag_num[6] == 0x6F) && (mag_num[7] == 0x74) ) return 7; // qt mov (pnot)
  if ( (mag_num[4] == 0x73) && (mag_num[5] == 0x6B) && (mag_num[6] == 0x69) && (mag_num[7] == 0x70) ) return 7; // qt mov (skip)
  if ( (mag_num[4] == 0x66) && (mag_num[5] == 0x74) && (mag_num[6] == 0x79) && (mag_num[7] == 0x70) && 
       (mag_num[8] == 0x71) && (mag_num[9] == 0x74) ) return 7; // qt mov (ftypqt)
  if ( (mag_num[24] == 0x77) && (mag_num[25] == 0x69) && (mag_num[26] == 0x64) && (mag_num[27] == 0x65) ) return 7; // qt mov (wide) futher

  if ( (mag_num[0] == 0x46) && (mag_num[1] == 0x4C) && (mag_num[2] == 0x56) ) return 8; // flash flv
  if ( (mag_num[0] == 0x46) && (mag_num[1] == 0x57) && (mag_num[2] == 0x53) ) return 8; // flash swf
  if ( (mag_num[0] == 0x43) && (mag_num[1] == 0x57) && (mag_num[2] == 0x53) ) return 8; // flash swf
  if ( (mag_num[0] == 0xD0) && (mag_num[1] == 0xCF) && (mag_num[2] == 0x11) && (mag_num[3] == 0xE0) &&
    (mag_num[4] == 0xA1) && (mag_num[5] == 0xB1) && (mag_num[6] == 0x1A) && (mag_num[7] == 0xE1) ) return 8; // fla

  if ( (mag_num[0] == 0x30) && (mag_num[1] == 0x26) && (mag_num[2] == 0xB2) && (mag_num[3] == 0x75) &&
    (mag_num[4] == 0x8E) && (mag_num[5] == 0x66) && (mag_num[6] == 0xCF) && (mag_num[7] == 0x11) &&
    (mag_num[8] == 0xA6) && (mag_num[9] == 0xD9) && (mag_num[10] == 0x00) && (mag_num[11] == 0xAA) &&
    (mag_num[12] == 0x00) && (mag_num[13] == 0x62) && (mag_num[14] == 0xCE) && (mag_num[15] == 0x6C) ) return 6; // wmv

  if ( (mag_num[0] == 0x00) && (mag_num[1] == 0x00) && (mag_num[2] == 0x00) && (mag_num[3] == 0x0C) &&
    (mag_num[4] == 0x6A) && (mag_num[5] == 0x50) && (mag_num[6] == 0x20) && (mag_num[7] == 0x20) &&
    (mag_num[8] == 0x0D) && (mag_num[9] == 0x0A) && (mag_num[10] == 0x87) && (mag_num[11] == 0x0A) ) return 4; // mj2

  // OGG Hex: 4F 67 67 53 ASCII: OggS
  if ( (mag_num[0] == 0x4F) && (mag_num[1] == 0x67) && (mag_num[2] == 0x67) && (mag_num[3] == 0x53) ) return 9; // OGG

  // Matroska  1A 45 DF A3
  if ( (mag_num[0] == 0x1A) && (mag_num[1] == 0x45) && (mag_num[2] == 0xDF) && (mag_num[3] == 0xA3) ) return 10; // Matroska

  // DV ?
  //if ( (mag_num[0] == 0x1A) && (mag_num[1] == 0x45) && (mag_num[2] == 0xDF) && (mag_num[3] == 0xA3) ) return 11; // DV

  // mpeg2 stream M2T
  if ( (mag_num[0] == 0x47) && (mag_num[1] == 0x08) && (mag_num[2] == 0x10) ) return 0; // RAW MPEG2 M2T

  return -1;
}

TDivFormatHandle __stdcall dimFFMpegAquireFormatProc( void ) {
  TDivFormatHandle fp = initTDivFormatHandle();
  fp.quality = 0;
  fp.resolutionLevel = 0;
  dimFFMpegLoadRegProc( &fp );

  // CODEC_ID_NONE indicates use of ffmpeg default codec
  // for some fomrats we would like to force the codec
  
  encoder_ids.push_back( CODEC_ID_NONE ); //encoder_ids.push_back( CODEC_ID_MPEG1VIDEO ); // 0 "MPEG",  - CODEC_ID_MPEG1VIDEO
  encoder_ids.push_back( CODEC_ID_NONE ); //encoder_ids.push_back( CODEC_ID_MPEG2VIDEO ); // 1 "MPEG2", - MPEG2VIDEO
  encoder_ids.push_back( CODEC_ID_NONE ); //encoder_ids.push_back( CODEC_ID_MPEG4 );      // 2 "MPEG4", - MPEG4
  encoder_ids.push_back( CODEC_ID_NONE ); //encoder_ids.push_back( CODEC_ID_MJPEG );      // 3 "MJEPG", - MJPEG
  encoder_ids.push_back( CODEC_ID_NONE ); //encoder_ids.push_back( CODEC_ID_NONE );       // 4 "MJEPG2000",
  encoder_ids.push_back( CODEC_ID_MPEG4 );      // 5 "AVI",   - XVID
  encoder_ids.push_back( CODEC_ID_NONE ); //encoder_ids.push_back( CODEC_ID_MSMPEG4V3 );  // 6 "WMV",   - WMV3
  encoder_ids.push_back( CODEC_ID_NONE ); //encoder_ids.push_back( CODEC_ID_MPEG4 );      // 7 "QT",
  encoder_ids.push_back( CODEC_ID_NONE ); //encoder_ids.push_back( CODEC_ID_FLV1 );       // 8 "FLASH",
  encoder_ids.push_back( CODEC_ID_NONE ); //encoder_ids.push_back( CODEC_ID_THEORA );     // 9 "Ogg format",
  encoder_ids.push_back( CODEC_ID_MPEG4 );      // 10 "Matroska File Format",
  encoder_ids.push_back( CODEC_ID_NONE ); //encoder_ids.push_back( CODEC_ID_DVVIDEO );    // 11 "DV video format",
  encoder_ids.push_back( CODEC_ID_NONE ); //encoder_ids.push_back( CODEC_ID_FLV1 );       // 12 "Flash Video",

  formats_write.push_back( "mpeg" );       // 0 "MPEG",  - MPEG1VIDEO
  formats_write.push_back( "mpeg2video" ); // 1 "MPEG2", - MPEG2VIDEO m2v
  formats_write.push_back( "mp4" );        // 2 "MPEG4", - MPEG4
  formats_write.push_back( "mjpeg" );      // 3 "MJEPG", - MJPEG
  formats_write.push_back( "mjpeg2000" );  // 4 "MJEPG2000",
  formats_write.push_back( "avi" );        // 5 "AVI",   - XVID
  formats_write.push_back( "asf" );        // 6 "WMV",   - WMV3
  formats_write.push_back( "mov" );        // 7 "QT", // "mov,mp4,m4a,3gp,3g2,mj2"
  formats_write.push_back( "swf" );        // 8 "FLASH",
  formats_write.push_back( "ogg" );        // 9 "Ogg format",
  formats_write.push_back( "matroska" );   // 10 "Matroska File Format",
  formats_write.push_back( "dv" );         // 11 "DV video format",
  formats_write.push_back( "flv" );        // 12 "FLV",

  formats_fourcc.push_back( "" );     // 0 "MPEG",  - MPEG1VIDEO
  formats_fourcc.push_back( "" );     // 1 "MPEG2", - MPEG2VIDEO m2v
  formats_fourcc.push_back( "" );     // 2 "MPEG4", - MPEG4
  formats_fourcc.push_back( "" );     // 3 "MJEPG", - MJPEG
  formats_fourcc.push_back( "" );     // 4 "MJEPG2000",
  formats_fourcc.push_back( "XVID" ); // 5 "AVI",   - XVID
  formats_fourcc.push_back( "" );     // 6 "WMV",   - WMV3
  formats_fourcc.push_back( "" );     // 7 "QT", // "mov,mp4,m4a,3gp,3g2,mj2"
  formats_fourcc.push_back( "" );     // 8 "FLASH",
  formats_fourcc.push_back( "" );     // 9 "Ogg format",
  formats_fourcc.push_back( "XVID" ); // 10 "Matroska File Format",
  formats_fourcc.push_back( "" );     // 11 "DV video format",
  formats_fourcc.push_back( "" );     // 12 "Flash video",

  formats.insert( std::make_pair( "mpeg", 0 ) );
  formats.insert( std::make_pair( "mpegvideo", 0 ) );
  formats.insert( std::make_pair( "mpeg1video", 0 ) );
  formats.insert( std::make_pair( "mpegts", 1 ) );
  formats.insert( std::make_pair( "mpegtsraw", 1 ) );
  formats.insert( std::make_pair( "mpeg2video", 1 ) );
  formats.insert( std::make_pair( "m4v", 2 ) );
  formats.insert( std::make_pair( "h264", 2 ) );
  formats.insert( std::make_pair( "mjpeg", 3 ) );
  formats.insert( std::make_pair( "ingenient", 3 ) );
  formats.insert( std::make_pair( "avi", 5 ) );
  formats.insert( std::make_pair( "asf", 6 ) );
  formats.insert( std::make_pair( "mov,mp4,m4a,3gp,3g2,mj2", 7 ) );
  formats.insert( std::make_pair( "swf", 8 ) );
  formats.insert( std::make_pair( "ogg", 9 ) );
  formats.insert( std::make_pair( "matroska", 10 ) );
  formats.insert( std::make_pair( "dv", 11 ) );
  formats.insert( std::make_pair( "flv", 12 ) );

  // if format case is ambiguous, use codecs
  codecs.insert( std::make_pair( "mpeg1video", 0 ) );
  codecs.insert( std::make_pair( "mpeg2video", 1 ) );
  codecs.insert( std::make_pair( "mpeg4", 2 ) );


  return fp;
}

void __stdcall dimFFMpegReleaseFormatProc (TDivFormatHandle *fmtHndl) {
  if (fmtHndl == NULL) return;
  dimFFMpegCloseImageProc ( fmtHndl );  
}


//----------------------------------------------------------------------------
// OPEN/CLOSE
//----------------------------------------------------------------------------

void dimFFMpegSetWriteParameters  (TDivFormatHandle *fmtHndl) {
  if (fmtHndl == NULL) return;
  if (fmtHndl->internalParams == NULL) return;  
  TDimFFMpegParams *par = (TDimFFMpegParams *) fmtHndl->internalParams;

  par->frame_sizes_set = false;
  par->fps = * (float*) &fmtHndl->resolutionLevel;
  par->bitrate = fmtHndl->quality;
}

void __stdcall dimFFMpegCloseImageProc (TDivFormatHandle *fmtHndl) {
  if (fmtHndl == NULL) return;
  if (fmtHndl->internalParams == NULL) return;  
  TDimFFMpegParams *par = (TDimFFMpegParams *) fmtHndl->internalParams;
  par->ff_in.close();
  par->ff_out.close();
  delete [] par;
  fmtHndl->internalParams = NULL;
}

DIM_UINT __stdcall dimFFMpegOpenImageProc  (TDivFormatHandle *fmtHndl, TDIVIOModes io_mode) {
  if (fmtHndl == NULL) return 1;
  if (fmtHndl->internalParams != NULL) dimFFMpegCloseImageProc (fmtHndl);  
  fmtHndl->internalParams = (void *) new TDimFFMpegParams [1];
  TDimFFMpegParams *par = (TDimFFMpegParams *) fmtHndl->internalParams;

  if ( io_mode == DIV_IO_READ ) {
    VideoIO::KeyValueMap kvm;
    std::string filename = fmtHndl->fileName;
    kvm["filename"] = filename;
    par->ff_in.setConvertToMatlab(false);
    par->ff_in.open( kvm );
    if ( !ffmpegGetImageInfo( fmtHndl ) ) { dimFFMpegCloseImageProc (fmtHndl); return 1; };
  }

  if ( io_mode == DIV_IO_WRITE ) {
    dimFFMpegSetWriteParameters(fmtHndl);
  }

  return 0;
}

//----------------------------------------------------------------------------
// INFO for OPEN image
//----------------------------------------------------------------------------

DIM_UINT __stdcall dimFFMpegGetNumPagesProc ( TDivFormatHandle *fmtHndl ) {
  if (fmtHndl == NULL) return 0;
  if (fmtHndl->internalParams == NULL) return 0;
  TDimFFMpegParams *par = (TDimFFMpegParams *) fmtHndl->internalParams;
  TDivImageInfo *info = &par->i;    
  return info->number_pages;
}


TDivImageInfo __stdcall dimFFMpegGetImageInfoProc ( TDivFormatHandle *fmtHndl, DIM_UINT page_num ) {
  TDivImageInfo ii = initTDivImageInfo();
  if (fmtHndl == NULL) return ii;
  TDimFFMpegParams *par = (TDimFFMpegParams *) fmtHndl->internalParams;
  return par->i;
}

//----------------------------------------------------------------------------
// METADATA
//----------------------------------------------------------------------------

DIM_UINT __stdcall dimFFMpegReadMetaDataProc (TDivFormatHandle *fmtHndl, DIM_UINT page, int group, int tag, int type) {
  if (fmtHndl == NULL) return 1;
  return 1;
}

char* __stdcall dimFFMpegReadMetaDataAsTextProc ( TDivFormatHandle *fmtHndl ) {
  if (fmtHndl == NULL) return NULL;
  if (fmtHndl->internalParams == NULL) return NULL;
  TDimFFMpegParams *par = (TDimFFMpegParams *) fmtHndl->internalParams;
  
  char cs[1024];
  sprintf(cs, "Format name: %s\nCodec name: %s\nFrames per second: %.4f\n", 
      par->ff_in.formatName(), par->ff_in.codecName(), par->ff_in.fps() );
  return cs;
}


//----------------------------------------------------------------------------
// READ/WRITE
//----------------------------------------------------------------------------

DIM_UINT __stdcall dimFFMpegReadImageProc  ( TDivFormatHandle *fmtHndl, DIM_UINT page ) {
  if (fmtHndl == NULL) return 1;
  fmtHndl->pageNumber = page;
  return read_ffmpeg_image( fmtHndl );
}

DIM_UINT __stdcall dimFFMpegWriteImageProc ( TDivFormatHandle *fmtHndl ) {
  if (fmtHndl == NULL) return 1;
  try {
    return write_ffmpeg_image( fmtHndl );
  } catch (...) {
    return 1;
  }
}



//****************************************************************************
// Registry - saving/loading parameters
//****************************************************************************

const char regPath[] = "SOFTWARE\\DIMIN\\Viewer5\\Formats\\FFMPEG\0";
const int buf_size = 4;

void __stdcall dimFFMpegLoadRegProc ( TDivFormatHandle *fmtHndl ) {
  HKEY hKey;
  LONG lRet;
  //TCHAR val[buf_size];
  DWORD bufLen = buf_size;
  int val;

  lRet = RegOpenKeyEx( HKEY_CURRENT_USER, regPath, 0, KEY_QUERY_VALUE, &hKey );
  
  if (lRet==ERROR_SUCCESS) {

    if ( RegQueryValueEx(hKey, TEXT("Bitrate"), NULL, NULL, (LPBYTE) &val, &bufLen) == ERROR_SUCCESS )
      fmtHndl->quality = val;

    if ( RegQueryValueEx(hKey, TEXT("fps"), NULL, NULL, (LPBYTE) &val, &bufLen) == ERROR_SUCCESS )
      fmtHndl->resolutionLevel = val;

  } // key open

  RegCloseKey( hKey );
}

void __stdcall dimFFMpegSaveRegProc ( TDivFormatHandle *fmtHndl ) {
  HKEY hKey;
  LONG lRet;
  DWORD dwDisp;
  int val;

  lRet = RegCreateKeyEx( HKEY_CURRENT_USER, regPath,
                         0, NULL, REG_OPTION_NON_VOLATILE,
                         KEY_WRITE, NULL, &hKey, &dwDisp );
  
  if (lRet==ERROR_SUCCESS) {

    val = fmtHndl->quality;
    RegSetValueEx( hKey, TEXT("Bitrate"), NULL, REG_DWORD, (const BYTE *) &val, 4);  

    val = fmtHndl->resolutionLevel;
    RegSetValueEx( hKey, TEXT("fps"), NULL, REG_DWORD, (const BYTE *) &val, 4);  

  } // key open

  RegCloseKey( hKey );
}


//****************************************************************************
// GUI DEMANDED FUNTIONS
//****************************************************************************

WNDPROC ffmpegWndProcOld; 
HWND ffmpegFPSCombo;

int bitrate = 0;
float fps = 0.0;

HWND fpsEditBox = 0;
HWND bitrateEditBox = 0;
HWND fpsLabel = 0;
HWND bitrateLabel = 0;

LRESULT CALLBACK ffmpegWndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
  BOOL Result = FALSE;

  xstring str;
  std::vector<char> buf(20, 0);
  int size = SendMessage( fpsEditBox, WM_GETTEXT, 20, (LPARAM) &buf[0] );
  str = &buf[0];
  fps = str.toDouble(); 

  buf.clear();
  buf.resize(20, 0);
  SendMessage( bitrateEditBox, WM_GETTEXT, 20, (LPARAM) &buf[0] );
  str = &buf[0];
  bitrate = str.toInt(); 

	if (Result != TRUE) 
    return CallWindowProc( ffmpegWndProcOld, hwnd, message, wParam, lParam);
  else 
    return Result;
}

void __stdcall dimFFMpegConfigInitProc ( TDivFormatHandle *fmtHndl, HWND hwndParent )
{
  HINSTANCE wHinst = (HINSTANCE) GetWindowLong( hwndParent, GWL_HINSTANCE );
  ffmpegWndProcOld = (WNDPROC) GetWindowLong( hwndParent, GWL_WNDPROC );
	SetWindowLong( hwndParent, GWL_WNDPROC, (long) ffmpegWndProc );
  

  HFONT hf;
  LOGFONT lf = { 0 };
    
  //GetObject( GetWindowFont(hwndParent), sizeof(lf), &lf );
  lf.lfWeight = FW_REGULAR;
  lf.lfHeight = (LONG)14;
  lstrcpy( lf.lfFaceName, "Tahoma" );

  hf=CreateFontIndirect(&lf);
  //SetBkMode(hdc,TRANSPARENT);
  SendMessage(hwndParent,WM_SETFONT,(WPARAM)hf,TRUE);


  // text label
  fpsLabel = CreateWindow("STATIC", "Frames per second (0-default):", 
     WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_EX_TRANSPARENT, // | SS_WHITERECT | WS_EX_TRANSPARENT,
     10, 15, 350, 20,
     hwndParent, NULL, wHinst, NULL);
  SendMessage(fpsLabel, WM_SETFONT, (WPARAM)hf, TRUE);

  // text label
  bitrateLabel = CreateWindow("STATIC", "Bitrate (0-default):", 
     WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS,// | WS_EX_TRANSPARENT,// | SS_WHITERECT,
     10, 40, 350, 20,
     hwndParent, NULL, wHinst, NULL);
  SendMessage(bitrateLabel, WM_SETFONT, (WPARAM)hf, TRUE);
 
  // edit box
  char str[128];
  sprintf( str, "%.2f", *(float*) &fmtHndl->resolutionLevel );
  //sprintf( str, "%.2f", 30.0 );
  fpsEditBox = CreateWindow("EDIT", str, 
     WS_CHILD | WS_VISIBLE | WS_TABSTOP | WS_BORDER | WS_CLIPSIBLINGS,
     360, 15, 50, 20,
     hwndParent, NULL, wHinst, NULL);

  // edit box
  sprintf( str, "%d", fmtHndl->quality );
  //sprintf( str, "%d", 0 );
  bitrateLabel = CreateWindow("EDIT", str, 
     WS_CHILD | WS_VISIBLE | ES_NUMBER | WS_TABSTOP | WS_BORDER | WS_CLIPSIBLINGS,
     360, 40, 50, 20,
     hwndParent, NULL, wHinst, NULL);
}

void __stdcall dimFFMpegConfigQuitProc ( TDivFormatHandle *fmtHndl, HWND hwndParent ) {
  fmtHndl->quality = bitrate;
  *(float*) &fmtHndl->resolutionLevel = fps; 

  SetWindowLong( hwndParent, GWL_WNDPROC, (long) ffmpegWndProcOld );
  if (fpsEditBox)     DestroyWindow( fpsEditBox );
  if (bitrateEditBox) DestroyWindow( bitrateEditBox );
  if (fpsLabel)       DestroyWindow( fpsLabel );
  if (bitrateLabel)   DestroyWindow( bitrateLabel );
  dimFFMpegSaveRegProc ( fmtHndl );
}

