/*****************************************************************************
  FFMPEG support 
  Copyright (c) 2008 by Dmitry V. Fedorov <www.dimin.net> <dima@dimin.net>

  IMPLEMENTATION
  
  Programmer: Dima V. Fedorov <mailto:dima@dimin.net> <http://www.dimin.net/>

  History:
    2008-02-01 14:45 - First creation
        
  Ver : 1
*****************************************************************************/

#include <string>
#include <algorithm>
#include "dim_ffmpeg_format.h"

//----------------------------------------------------------------------------
// READ PROC
//----------------------------------------------------------------------------

static int read_ffmpeg_image(TDivFormatHandle *fmtHndl) {
  if (fmtHndl == NULL) return 1;
  if (fmtHndl->internalParams == NULL) return 1;
  TDimFFMpegParams *par = (TDimFFMpegParams *) fmtHndl->internalParams;
  TDivImageInfo *info = &par->i;  

  //allocate image
  int depth = 24;
  unsigned char *p; // pointer to image data
  RGBQUAD lut[256];
  if (dimCreateHBitmap( fmtHndl, info->width, info->height, depth, 0, lut, &p ) != 0) return 1;

  // init the image
  unsigned int page_size = info->width * info->height * info->samples;
  unsigned int line_size = info->width * info->samples;

  bool res = false;
  try {
    res = par->ff_in.seek(fmtHndl->pageNumber);
  } catch (...) {
    return 1;
  }

  if ( res ) {
    const std::vector<unsigned char> *frame = par->ff_in.currBGR();
    const unsigned char *fbuf = &(*frame)[0];

    if (frame->size() >= page_size) {
      const unsigned char *pI = fbuf;// + (info->samples-c-1);

      for ( int y=0; y<info->height; ++y ) {
        showProgress( fmtHndl, y, info->height, "Reading Video" );
        if ( testAbort( fmtHndl ) == 1) break;   
        DIM_UCHAR *pix = getHBScanLine( info->height-y-1, p, info->width, depth );
        memcpy( pix, fbuf, line_size );
        fbuf += line_size;
      } // for y
    }
  } // if could seek to the page

  return 0;
}

static int write_ffmpeg_image(TDivFormatHandle *fmtHndl) {
  if (fmtHndl == NULL) return 1;
  if (fmtHndl->internalParams == NULL) return 1;
  TDimFFMpegParams *par = (TDimFFMpegParams *) fmtHndl->internalParams;
  TDivImageInfo *info = &par->i;  

  BITMAP bmp;
  // Retrieve the bitmap color format, width, and height.
  GetObject( fmtHndl->hbmp, sizeof(BITMAP), (LPSTR)&bmp );
  unsigned char *p = (unsigned char *) bmp.bmBits;
  if ( bmp.bmBitsPixel != 24 ) return 1;

  info->width = bmp.bmWidth;
  info->height = bmp.bmHeight;
  info->samples = 3;

  if (!par->frame_sizes_set) {

    std::string filename = fmtHndl->fileName;
    par->ff_out.setWidth( info->width );
    par->ff_out.setHeight( info->height );
    par->ff_out.setFormat( formats_write[ fmtHndl->subFormat ] );

    // use our default codec
    par->ff_out.setCodec( encoder_ids[ fmtHndl->subFormat ] );

    // for some formats set fourcc to non default one, e.g. XVID instead of default FMP4 for better compatibility of AVI
    par->ff_out.setFourCC( formats_fourcc[ fmtHndl->subFormat ] );

    if (par->fps > 0) par->ff_out.setFramesPerSecond(par->fps);
    if (par->bitrate > 0) par->ff_out.setBitRate(par->bitrate);

    VideoIO::KeyValueMap kvm;
    kvm["filename"] = filename;
    kvm["depth"]    = "3";
    
    par->ff_out.setup( kvm );    
    //par->ff_out.open( filename );
    if (!par->ff_out.isOpen()) return 1;
    par->frame_sizes_set = true;
  }

  unsigned char *fbuf = par->ff_out.rawFrameData();
  unsigned char *pI = fbuf;
  unsigned int line_size = info->width * info->samples;

  for ( int y=0; y<info->height; ++y ) {

    showProgress( fmtHndl, y, info->height, "Writing video" );
    if ( testAbort( fmtHndl ) == 1) return 1;      
    DIM_UCHAR *pix = getHBScanLine( info->height-y-1, p, bmp.bmWidth, bmp.bmBitsPixel );
    //memcpy( fbuf, pix, line_size );
    for (int x=0; x<bmp.bmWidth*3; x+=3) {
      fbuf[x+0] = pix[x+2];
      fbuf[x+1] = pix[x+1];
      fbuf[x+2] = pix[x+0];
    }
    fbuf += line_size;

  } // for y

  try {
    par->ff_out.addFromRawFrame( info->width, info->height, 3 );
  } catch (...) {
    return 1;
  }

  return 0;
}

